<?php

namespace FiloBlu\ExtInventory\Helper;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\IsProductSalableInterface;
use Magento\InventorySalesApi\Api\StockResolverInterface;
use Magento\InventorySales\Model\ResourceModel\StockIdResolver;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Store\Model\ResourceModel\Website\CollectionFactory;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Data
 * @package FiloBlu\ExtInventory\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var IsProductSalableInterface
     */
    protected $isProductSalableInterface;
    /**
     * @var StockResolverInterface
     */
    protected $stockResolver;
    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var CollectionFactory
     */
    protected $websiteCollectionFactory;
    /**
     * @var StockIdResolver
     */
    protected $stockIdResolver;
    /**
     * @var StoreManagerInterface
     */
    public $storeManager;
    /**
     * @var OrderRepositoryInterface
     */
    public $orderRepositoryInterface;

    /**
     * Data constructor.
     * @param Context $context
     * @param IsProductSalableInterface $isProductSalableInterface
     * @param StockResolverInterface $stockResolver
     * @param CollectionFactory $websiteCollectionFactory
     * @param ProductRepository $productRepository
     * @param StockIdResolver $stockIdResolver
     * @param StoreManagerInterface $storeManager
     * @param OrderRepositoryInterface $orderRepositoryInterface
     */
    public function __construct(
        Context $context,
        IsProductSalableInterface $isProductSalableInterface,
        StockResolverInterface $stockResolver,
        CollectionFactory $websiteCollectionFactory,
        ProductRepository $productRepository,
        StockIdResolver $stockIdResolver,
        StoreManagerInterface $storeManager,
        OrderRepositoryInterface $orderRepositoryInterface
    )
    {
        parent::__construct($context);
        $this->isProductSalableInterface = $isProductSalableInterface;
        $this->stockResolver = $stockResolver;
        $this->websiteCollectionFactory = $websiteCollectionFactory;
        $this->productRepository = $productRepository;
        $this->stockIdResolver = $stockIdResolver;
        $this->storeManager = $storeManager;
        $this->orderRepositoryInterface = $orderRepositoryInterface;
    }

    /**
     * @return bool
     */
    public function isEnabled(): bool
    {
        return true;
    }

    /**
     * @param $productSku
     * @param $websiteId
     * @return bool
     * @throws NoSuchEntityException
     */
    public function isSalable($productSku, $websiteId): bool
    {
        //$product = $this->getProductById($productId);
        $websiteCode = $this->getWebsiteCodeById($websiteId);
        $stockId = $this->getStockIdByWebsiteCode($websiteCode);

        if (is_object($productSku)) {
            $productSku = $productSku->getSku();
        }

        return $this->isProductSalableInterface->execute($productSku, $stockId);
    }

    /**
     * @param $websiteId
     * @return mixed
     */
    public function getWebsiteCodeById($websiteId)
    {
        $collection = $this->websiteCollectionFactory->create();
        $collection->addFieldToFilter('website_id', ['eq' => $websiteId]);
        $websiteData = $collection->getData();
        return $websiteData[0]['code'];
    }

    /**
     * @param $website_code
     * @return int|null
     * @throws NoSuchEntityException
     */
    public function getStockIdByWebsiteCode($website_code)
    {
        $stock = $this->stockResolver->execute(SalesChannelInterface::TYPE_WEBSITE, $website_code);
        return $stock->getStockId();
    }

    /**
     * @param $order
     * @return int|null
     * @throws NoSuchEntityException
     */
    public function getStockIdByOrder(OrderInterface $order){
        $websiteId = $this->storeManager->getStore($order->getStoreId())->getWebsiteId();
        $websiteCode = $this->storeManager->getWebsite($websiteId)->getCode();
        $stockId = $this->stockIdResolver->resolve(SalesChannelInterface::TYPE_WEBSITE, $websiteCode);
        return $stockId;
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getCurrentStoreviewId(){
        $currentStore = $this->storeManager->getStore();
        return $currentStore->getId();
    }

    /**
     * @param $id
     * @return ProductInterface|mixed
     * @throws NoSuchEntityException
     */
    public function getProductById($id)
    {
        return $this->productRepository->getById($id);
    }

    /**
     * @param $sku
     * @return ProductInterface|Product|null
     * @throws NoSuchEntityException
     */
    public function getProductBySku($sku)
    {
        return $this->productRepository->get($sku);
    }

    /**
     * @param $order_id
     * @return int|null
     */
    public function getOrderStoreviewIdByOrderId($order_id){
        $order = $this->orderRepositoryInterface->get($order_id);
        return $order->getStoreId();
    }
}